package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"encoding/json"
	"errors"
	"fmt"
	"time"
	"xorm.io/xorm"
)

// SettlementSignInGreenEnergy 计算签到得到绿色能量
func SettlementSignInGreenEnergy(engine *xorm.Engine, masterId string) (err error) {
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	var teamRewardSetting *md2.TeamRewardSettingStruct
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.VipEquitySetting), &teamRewardSetting)
	if err != nil {
		return
	}
	if teamRewardSetting.RewardDecrementValue == "" || teamRewardSetting.RewardEndValue == "" || teamRewardSetting.MemberSelfIsOpenGetTeamReward == "" || teamRewardSetting.OneRoundDuration == "" {
		err = errors.New("团队奖励设置未完全！")
		return
	}
	var oneRoundDuration = zhios_order_relate_utils.StrToInt(teamRewardSetting.OneRoundDuration)
	var rewardDecrement = zhios_order_relate_utils.StrToFloat64(teamRewardSetting.RewardDecrementValue) / 100 //递减百分比
	var rewardEndValue = zhios_order_relate_utils.StrToFloat64(teamRewardSetting.RewardEndValue)              //奖励结束值
	fmt.Println("rewardDecrement>>>>>>>>>>>>", rewardDecrement)
	fmt.Println("rewardEndValue>>>>>>>>>>>>", rewardEndValue)
	var signInRewards []*md2.SignInRewardStruct
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.VipEquitySetting), &signInRewards)
	if err != nil {
		return
	}
	if len(signInRewards) == 0 {
		err = errors.New("未设置签到奖励！")
		return
	}

	//2、统计全网用户数
	sqlStr := "SELECT COUNT(*) AS total FROM USER "
	nativeString, _ := db.QueryNativeString(engine, sqlStr)
	userCount := zhios_order_relate_utils.StrToInt64(nativeString[0]["total"])

	var rewardValue string
	for _, v := range signInRewards {
		if zhios_order_relate_utils.StrToInt64(v.VipMemberEndNums) <= userCount && userCount <= zhios_order_relate_utils.StrToInt64(v.VipMemberStartNums) {
			rewardValue = zhios_order_relate_utils.Float64ToStrPrec8(zhios_order_relate_utils.StrToFloat64(v.RewardValue) * float64(oneRoundDuration) * 60 * 60)
		}
	}

	//3、统计签到结束的用户数据
	now := time.Now()
	startTime := now.Add(-time.Hour * time.Duration(oneRoundDuration)).Format("2006-01-02 15:04:05")
	var list []model.OneCirclesGreenEnergySignIn
	err = engine.Where("start_time <= ?", startTime).And("is_completed =?", 0).Find(&list)
	if err != nil {
		fmt.Println("err:::::1111", err)
		return
	}

	session := engine.NewSession()
	var reduceTotalGreenEnergy float64
	for _, v := range list {
		//4.2给相应的用户加上个人的绿色积分(可用数量)
		err = DealUserCoin(session, md.DealUserCoinReq{
			Kind:         "add",
			Mid:          masterId,
			Title:        md.OneCirclesGreenEnergySignInSettlementPersonalReward,
			TransferType: md.OneCirclesGreenEnergySignInSettlementPersonalRewardForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       oneCirclesGreenEnergyBasicSetting.PersonGreenEnergyCoinId,
			Uid:          v.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(rewardValue),
		})
		if err != nil {
			_ = session.Rollback()
			fmt.Println("err:::::2222", err)
			return err
		}
		reduceTotalGreenEnergy += zhios_order_relate_utils.StrToFloat64(rewardValue)

		//4.2给相应的上级用户加上团队奖励的绿色积分(结算数量)
		relates, err1 := db.DbsUserRelate(engine, v.Uid, 0)
		if err1 != nil {
			_ = session.Rollback()
			fmt.Println("err:::::3333", err1)
			return err1
		}
		var parentRewardValue = zhios_order_relate_utils.StrToFloat64(rewardValue)
		for _, relate := range *relates {
			if parentRewardValue <= rewardEndValue {
				break
			}
			parentRewardValue = parentRewardValue * rewardDecrement
			err = DealUserCoin(session, md.DealUserCoinReq{
				Kind:         "add",
				Mid:          masterId,
				Title:        md.OneCirclesGreenEnergySignInSettlementTeamReward,
				TransferType: md.OneCirclesGreenEnergySignInSettlementTeamRewardForUserVirtualCoinFlow,
				OrdId:        "",
				CoinId:       oneCirclesGreenEnergyBasicSetting.TeamGreenEnergyCoinId,
				Uid:          relate.Uid,
				Amount:       parentRewardValue,
			})
			if err != nil {
				_ = session.Rollback()
				fmt.Println("err:::::44444", err)
				return err
			}
			reduceTotalGreenEnergy += parentRewardValue
		}
	}

	//5、减少“活跃赠送” 中的绿色能量
	err = DealAvailableGreenEnergyCoin(session, int(enum.SignInReward), reduceTotalGreenEnergy, enum.SignInReward.String(), oneCirclesGreenEnergyBasicSetting)
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err:::::55555", err)
		return err
	}
	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}
	return
}
